import { useState } from 'react'

import { gotError, gotResponse } from '../hooks/useFetch'
import { useUserRating, useSendRate } from '../hooks/api'

import styles from '../styles/Rating.module.css'

type StarProps = {
    filled: boolean,
    selected: boolean,
    setMyRate: () => void,
    sendMyRate: () => void,
}

function Star({ filled, selected, setMyRate, sendMyRate }: StarProps) {
    return (
        <button
            className={`${styles.star} ${filled ? styles.starFilled : ''} ${selected ? styles.starSelected : ''}`}
            onMouseEnter={setMyRate}
            onFocus={setMyRate}
            onClick={sendMyRate}
        >&#9733;</button>
    )
}

type RatingProps = {
    userId: number,
    className: string | undefined,
}

function Rating({ userId, className }: RatingProps) {
    const rating = useUserRating(userId)

    const [myRate, setMyRate] = useState(0)

    const { doSendRate } = useSendRate()

    async function sendMyRate() {
        const res = await doSendRate(myRate)

        if (res) {
            rating.refetch()
        }
    }

    return (
        <p className={className}>
            Рейтинг пользователя:{' '}
            {gotResponse(rating) ? (
                gotError(rating) ? (
                    <span className='text-danger'>{rating.error}</span>
                ) : (
                    <span
                        className={styles.starContainer}
                        onMouseLeave={() => setMyRate(0)}
                    >
                        {...Array(5).fill(5).map(
                            (_, i) =>
                                <Star
                                    key={i}
                                    filled={i < rating.data}
                                    selected={i < myRate}
                                    setMyRate={() => setMyRate(i + 1)}
                                    sendMyRate={() => void sendMyRate()}
                                />
                        )}
                    </span>
                )
            ) : (
                <span>Загрузка...</span>
            )
            }
        </p>

    )
}

export default Rating
